<?php

namespace Beo\Story\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Beo\Story\Forms\Category as CategoryForm;
use Beo\Story\Http\Requests\CategoryRequest;
use Beo\Story\Http\Resources\CategoryResource;
use Beo\Story\Http\Resources\CategoriesCollection;
use Beo\Story\Models\Category;
use Beo\Story\Scopes\CategoryResourceScope;


/**
 * Resource controller class for category.
 */
class CategoryResourceController extends BaseController
{

    /**
     * Initialize category resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = CategoryForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('beo.story.modules'), 'story', guard_url('story'));
    }

    /**
     * Display a list of category.
     *
     * @return Response
     */
    public function index(CategoryRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Category::pushScope(new RequestScope())
            ->pushScope(new CategoryResourceScope())
            ->simplePaginate($pageLimit);

        $data = new CategoriesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('story::category.names'))
            ->view('story::category.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display category.
     *
     * @param Request $request
     * @param Model   $category
     *
     * @return Response
     */
    public function show(CategoryRequest $request, Category $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CategoryResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('story::category.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('story::category.show')
            ->output();

    }

    /**
     * Show the form for creating a new category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CategoryRequest $request, Category $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CategoryResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('story::category.name'))
            ->view('story::category.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CategoryRequest $request, Category $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new CategoryResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('story::category.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('story/category/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/story/category'))
                ->redirect();
        }

    }

    /**
     * Show category for editing.
     *
     * @param Request $request
     * @param Model   $category
     *
     * @return Response
     */
    public function edit(CategoryRequest $request, Category $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CategoryResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('story::category.name'))
            ->view('story::category.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the category.
     *
     * @param Request $request
     * @param Model   $category
     *
     * @return Response
     */
    public function update(CategoryRequest $request, Category $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new CategoryResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('story::category.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('story/category/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('story/category/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the category.
     *
     * @param Model   $category
     *
     * @return Response
     */
    public function destroy(CategoryRequest $request, Category $model)
    {
        try {
            $model->delete();
            $data = new CategoryResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('story::category.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('story/category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('story/category/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
